unit DAVSL;
{-----------------------------------------------------------------
Part of Diagramantics VSL Framework (DAVSL)
www.diagramantics.com

This unit supplies:
-- A TVSL class (and singleton object VSL) to manage a Visio VSL
-- A base TAddOn class for creating individual AddOn modules.
  (Note that a VSL can contain one or more AddOn modules.)

NOTE: THIS UNIT NEED NOT BE MODIFIED FOR MOST VSL PROJECTS -- JUST INCLUDE
IN PROJECT AS IS.  

Sample RUNADDON calls:
----------------------
Action = RUNADDONWARGS("DemoAddOn_Form01","Fred=2 Joe=5")
Action = RUNADDON("DemoAddOn_Form01")

Argument parser has following features:
---------------------------------------
  Assumes args are keyword=value pairs, sep by spaces
  Parses into items of a TStringList
  Upper-cases the keyword so can reliably use TStringList.Values[]
  Handles quoted strings
    eg: RUNADDONWARGS("SomeAddOn","AnArgWSpaces=""Arg with spaces"" ")

History
--------
2000-10-05 GW Fixed bug in argument parser which missed final argument
2000-09-23 GW Added quote-handling
2000-08-28 GW Initial versions

------------------------------------------------------------------}

//====================================================
                    interface
//====================================================

uses classes, Visio_TLB, DAVAO, DAVSL_Util;

Const
  AddOnMax = 30; // max number of addons, increase if needed.

Type
  //---------------------
  TAddOn = class
  //---------------------
  protected
    // AddOnId     : word;
    FName       : string;
    Descr       : string;
    RegRec      : TVAORegRecord;
    Info        : TAddOnInfo;

    constructor Create(
      atts: TVAO_Atts;                // Attribs: VAO_AOATTS_xxx
                                      // ISACTION | HASABOUT | HASHELP | WAITCRSR |DONTSHOW
      enabMask: TVAO_EnabMask;        // Add-on's enabling policy. VAO_ENABLExxx
                                      // ALWAYS / DYNAMIC / DYNAMICEX, OR with
                                      // VAO_NEEDSxxx constants (see VAO or GWVAO)
      invokeOnMask: TVAO_InvokeMask;  // Invoke conds, 0 or   VAO_INVOKE_LAUNCH
      notifyOnMask: TVAO_NotifyMask;  // notify conditions (none?)
      AName       : String;           // Add-on's name
      ADescr      : String            // Add-on's description
    ); virtual;

    // destructor Destroy;

    // property Name: string read FName;
    // property Ix  : word read RegRec.ord;

    //------------------------------------
    // Called from VisioLibMain
    //------------------------------------
    function vlm_LOAD        (InterfaceVersion: word; visApp: Visio_TLB.Application): TVAORC; virtual;
    function vlm_UNLOAD                              : TVAORC; virtual;
    function vlm_ENUMADDONS  (V2LRec: PVAOV2LRecord   ): TVAORC; virtual;
    function vlm_ISAOENABLED (V2LRec: PVAOV2LRecord   ): TVAORC; virtual;
    function vlm_RUN         (V2LRec: PVAOV2LRecord;  Args: TStringList): TVAORC; virtual;
    function vlm_RUNABOUT    (V2LRec: PVAOV2LRecord   ): TVAORC; virtual;
    function vlm_RUNHELP     (V2LRec: PVAOV2LRecord   ): TVAORC; virtual;
    function vlm_KILLSESSION (V2LRec: PVAOV2LRecord   ): TVAORC; virtual;

    //------------------------------------
    // Utility
    //------------------------------------
    procedure ErrorMessage(Msg: string);
  public
    property Name: string read FName;
    procedure InfoStr(var OutputStr: string; IndentSpaces: integer);
  end;

  //---------------------
  TVSL = class
  //---------------------
  private
    FVisioCallBack: TVAOFunc;
    FvisApp       : Visio_TLB.Application;
    FAddOns       : array[1..AddOnMax] of TAddOn;
    FAppVerInfo   : TAppVerInfo;

    procedure SetVisioCallBack(f: TVAOFunc);
    procedure InitAutomation;
    property VisioCallBack: TVAOFunc read FVisioCallBack;
    function  RegisterAddon(wSessID: word; ARegRec: PVAORegRecord): TVAORC;
  protected
    //-----------------------------------------------------
    // VisioLibMain individual functions, can be overridden
    // (In some cases, descendants should call ancestor.)
    //-----------------------------------------------------
    function vlm_LOAD        (InterfaceVersion: word; VisioCallbackFunc: pointer): TVAORC; virtual;
    function vlm_UNLOAD      (UndefinedW: word; UndefinedP: pointer): TVAORC; virtual;
    function vlm_ENUMADDONS  (UndefinedW: word; VAO2LStruc: pointer): TVAORC; virtual;
    function vlm_ISAOENABLED (AddOnId: word; V2LRec: pointer): TVAORC; virtual;
    function vlm_RUN         (AddOnId: word; V2LRec: pointer): TVAORC; virtual;
    function vlm_RUNABOUT    (AddOnId: word; V2LRec: pointer): TVAORC; virtual;
    function vlm_RUNHELP     (AddOnId: word; V2LRec: pointer): TVAORC; virtual;
    function vlm_KILLSESSION (Zero   : word; V2LRec: pointer): TVAORC; virtual;

  public
    property visApp: Visio_TLB.Application read FvisApp;
    property AppVerInfo: TAppVerInfo read FAppVerInfo;

    constructor Create; virtual;
    destructor Destroy; override;

    function VisioLibMain(wMsg: TVAOMsg; wParam: word; lpParam: pointer): TVAORC;
    function EnlistAddOn(AddOn: TAddOn): word; // returns AddOnId

    procedure StandardAboutDialog;
    procedure InfoStr(var OutputStr: string; IndentSpaces: integer);

    procedure SendEndSession(wSessID: word); cdecl;
  end;

Var
  VSL:  TVSL;

Const
  VisioClassName   = 'visio.application';

//====================================================
                 implementation
//====================================================
uses
  Windows, ComObj,
  Dialogs,
  Forms,
  SysUtils,
  DAVSL_About;

//------------------------------------------------
function CheckLimits(CheckNum, AMin, AMax: integer): boolean;
//------------------------------------------------
Begin
  Result := (CheckNum >= AMin) and (CheckNum <= AMax);
end;


//---------------------------
constructor TVSL.Create;
//---------------------------
Var
  Ix: integer;
Begin
  FvisApp := nil;
  For Ix := 1 to AddOnMax do
  Begin
    FAddOns[Ix] := nil;
  end;
  FAppVerInfo := TAppVerInfo.Create;
end;

//---------------------------
destructor TVSL.Destroy;
//---------------------------
begin
  FvisApp := nil;
  inherited;
end;

//-----------------------------------------------------------------------------
function TVSL.RegisterAddon(wSessID: word; ARegRec: PVAORegRecord): TVAORC;
//-----------------------------------------------------------------------------
begin
  result := VAORC_SUCCESS;
  if Assigned(VisioCallBack) then
    result := VisioCallBack(L2VMSG_REGADDON, wSessID, ARegRec);
end;

//-----------------------------------------------------------------------------
procedure TVSL.SendEndSession(wSessID: word);
//-----------------------------------------------------------------------------
begin
  if Assigned(VisioCallBack) then
    VisioCallback(L2VMSG_SESSIONDONE, wSessID, nil);
end;

//------------------------------------------------
procedure TVSL.SetVisioCallBack(f: TVAOFunc);
//------------------------------------------------
begin
  fVisioCallBack := f;
end;

//------------------------------------------------
procedure TVSL.InitAutomation;
//------------------------------------------------
Var
  MyIDispatch: IDispatch;
  hApp: THandle;
Begin
  if not Assigned(FvisApp) then
  Begin
    try
      MyIDispatch := GetActiveOLEObject(VisioClassName);
      FvisApp      := MyIDispatch as IVApplication;
      if Assigned(visApp) then
      Begin
        //-------------------------------------------------------------------
        // We copy Visio's handle so that windows (TForms etc) of this
        // VSL behave like windows of Visio and don't show up separately
        // on the Taskbar.
        //-----------------------------------------
        hApp := visApp.WindowHandle32;
        Application.Handle := hApp;
      end;
    except
      ; // do nothing except catch exception for no instance running
    end;
  end;
  {--------------------------------------------------
    Ravelin had code here to check:
    -- Agreement of InstanceHandle32: but that's not useful since
       InstanceHandle 32 is always the same.
    -- Agreement of VAOUtil_VisCallBack<>pvlRunInfo^.lpfunc
       I think that is overkill.
   --------------------------------------------------}
end;

//------------------------------------------------
function TVSL.EnlistAddOn(AddOn: TAddOn): word;
//------------------------------------------------
// Returns AddOnId, or zero if failed
Var
  AddOnId: word;
Label the_end;
begin
  Result := 0;
  try
    For AddOnId := 1 to AddOnMax do
    Begin
      If not Assigned(FAddOns[AddOnId]) then
      Begin
        FAddOns[AddOnId] := AddOn;
        Result := AddOnId;
        goto the_end;
      end;
    end;
  the_end:
  except
  end;
end;

//------------------------------------------------------
procedure TVSL.StandardAboutDialog;
//------------------------------------------------------
Var
  dlg_About: Tdlg_About;
  S: string;
begin
  dlg_About := Tdlg_About.Create(Forms.Application);
  dlg_About.Memo1.Clear;

  S := '';
  VSL.InfoStr(S, 0);
  dlg_About.Memo1.Lines.Add(S);
  dlg_About.ShowModal;
end;

//------------------------------------------------------
procedure TVSL.InfoStr(var OutputStr: string; IndentSpaces: integer);
//------------------------------------------------------
Var
  AddOnId: word;
begin
  OutputStr := OutputStr
    + StringOfChar(' ', IndentSpaces) + 'VSL: ' + AppVerInfo.VSLName + CRLF
    + StringOfChar(' ', IndentSpaces) + AppVerInfo.FilePath + CRLF
    + StringOfChar(' ', IndentSpaces+2) + 'Company Name    : '+AppVerInfo.CompanyName     + CRLF
    + StringOfChar(' ', IndentSpaces+2) + 'File Description: '+AppVerInfo.FileDescription + CRLF
    + StringOfChar(' ', IndentSpaces+2) + 'Version         : '+AppVerInfo.FileVersion     + CRLF
    + StringOfChar(' ', IndentSpaces+2) + 'Legal Copyright : '+AppVerInfo.LegalCopyright  + CRLF
  ;

  For AddOnId := 1 to AddOnMax do
  Begin
    If Assigned(FAddOns[AddOnId]) then
      FAddOns[AddOnId].InfoStr(OutputStr, IndentSpaces);
  end;
end;

//-----------------------------------------------------------------------------
function TVSL.VisioLibMain(wMsg: TVAOMsg; wParam: word; lpParam: pointer): TVAORC;
//-----------------------------------------------------------------------------
Begin
  case wMsg of
    V2LMSG_LOAD       : Result := vlm_LOAD        (wParam, lpParam);
    V2LMSG_UNLOAD     : Result := vlm_UNLOAD      (wParam, lpParam);
    V2LMSG_ENUMADDONS : Result := vlm_ENUMADDONS  (wParam, lpParam);
    V2LMSG_ISAOENABLED: Result := vlm_ISAOENABLED (wParam, lpParam);
    V2LMSG_RUN        : Result := vlm_RUN         (wParam, lpParam);
    V2LMSG_RUNABOUT   : Result := vlm_RUNABOUT    (wParam, lpParam);
    V2LMSG_RUNHELP    : Result := vlm_RUNHELP     (wParam, lpParam);
    V2LMSG_KILLSESSION: Result := vlm_KILLSESSION (wParam, lpParam);
  else
    result := VAORC_L2V_BADMSG;
  end; // case
end;

//------------------------------------------------
function TVSL.vlm_ENUMADDONS(UndefinedW: word; VAO2LStruc: pointer): TVAORC;
//------------------------------------------------
Var
  AddOnId: Word;
begin
  try
    For AddOnId := 1 to AddOnMax do
    Begin
      If Assigned(FAddOns[AddOnId]) then
      Begin
        FAddOns[AddonId].vlm_ENUMADDONS(VAO2LStruc);
      end;
    end;
  except
  end;

  result := VAORC_SUCCESS;
end;

//------------------------------------------------
function TVSL.vlm_ISAOENABLED(AddOnId: word; V2LRec: pointer): TVAORC;
//------------------------------------------------
// proper response: VAORC_L2V_ENABLED | 0
begin
  Result := 0;
  try
    If CheckLimits(AddOnId,1,AddOnMax) and Assigned(FAddOns[AddOnId]) then
    Begin
      Result := FAddOns[AddOnId].vlm_ISAOENABLED(V2LRec);
    end;
  except
  end;
end;

//------------------------------------------------
function TVSL.vlm_KILLSESSION(Zero   : word; V2LRec: pointer): TVAORC;
//------------------------------------------------
// proper response: moot
Var
  AddOnId: Word;
begin
  try
    For AddOnId := 1 to AddOnMax do
    Begin
      If Assigned(FAddOns[AddOnId]) then
      Begin
        FAddOns[AddOnId].vlm_KILLSESSION(V2LRec);
      end;
    end;
  except
  end;
  result := VAORC_SUCCESS;
end;

//------------------------------------------------
function TVSL.vlm_LOAD(InterfaceVersion: word; VisioCallbackFunc: pointer): TVAORC;
//------------------------------------------------
// proper response: VAO_L2V_VERSION() | VAORC_L2V_LOADFAILED
var
  pfCB: PVAOFunc;
  AddOnId: word;
Label the_end;
begin
  result := VAO_L2V_VERSION;
  pfCB := @FVisioCallBack;
  if not Assigned(pfCB) then
  Begin
    SetVisioCallBack(VisioCallbackFunc);
    InitAutomation;
  end else
  Begin
    if pfCB<>@VisioCallBack then
    Begin
      result := VAORC_L2V_LOADFAILED;
      goto the_end;
    end;
  end;
  //---------------------------------------
  // Give Individual AddOns a chance
  //---------------------------------------
  try
    For AddOnId := 1 to AddOnMax do
    Begin
      If Assigned(FAddOns[AddOnId]) then
      Begin
        FAddOns[AddOnId].vlm_LOAD(InterfaceVersion, visApp);
      end;
    end;
  except

  end;
the_end:
end;

//------------------------------------------------
function TVSL.vlm_RUN(AddOnId: word; V2LRec: pointer): TVAORC;
//------------------------------------------------
// proper response: VAORC_SUCCESS | VAORC_L2V_MODELESS | VAORC_FAILURE
  //...................................................
  procedure ParseArgs(ArgStr: string; SL: TStringList);
  //...................................................
  Var
    S: string;
    ParseState : (ps_None, ps_InKey, ps_AfterKey, ps_InVal);

    procedure EnList;
    Begin
      SL.Add(S);
      S := '';
      ParseState := ps_None;
    end;

  Var
    P, L, N : integer;
    Quoted     : boolean;
    C, NextC: char;
  Begin
    L           := length(ArgStr);
    P           := 0;
    ParseState  := ps_None;
    S           := '';
    C           := ' ';
    while (P < L) do
    Begin
      Inc(P);
      C := ArgStr[P];
      If P < L then NextC := ArgStr[P+1] else NextC := ' ';
      Case ParseState of
        ps_None: Begin
          Case C of
            ' ', '/': ; // do nothing
            else Begin
              S := upcase(C);
              ParseState := ps_InKey;
            end;
          end; // case
        end;
        ps_InKey: Begin
          Case C of
            ' ': Enlist; // space after key means no value follows
            '=': Begin        // value follows, continue
              S := S + C;
              ParseState := ps_AfterKey;
            end;
            else S := S + upcase(C);
          end;
        end;
        ps_AfterKey: Begin
          Case C of
            ' ': ; // do nothing
            '"': Begin
              ParseState := ps_InVal;
              Quoted := true;
            end;
            else Begin
              S := S + C;
              ParseState := ps_InVal;
              Quoted := false;
            end;
          end;
        end;
        Ps_InVal: Begin
          Case C of
            ' ': if Quoted then S := S + C else Enlist;
            '"': Begin
              If Quoted then Enlist else S := S + C;
            end;
            else S := S + C;
          end;  // case
        end;
      end; // case
    end; // while
    if S <> '' then Enlist;
  end; // proc

Var
  SL: TStringList;
  ArgStr, S: string;
begin
  result := VAORC_FAILURE;
  //---------------------------------------
  // Dispatch to appropriate AddOn
  //---------------------------------------
  try
    If CheckLimits(AddOnId,1,AddOnMax) and Assigned(FAddOns[AddOnId]) then
    Begin
      SL := TStringList.Create;
      ArgStr := PVAOV2LRecord(V2LRec)^.lpCmdLineArgs;
      ParseArgs(ArgStr, SL);
      result := FAddOns[AddOnId].vlm_RUN(V2LRec, SL);
      SL.Free;
    end else
    Begin
      MessageDlg('Attempt to Run AddOn ' + IntToStr(AddOnId), mtInformation, [mbOk], 0);
      result := VAORC_SUCCESS;
    end;
  except
  end;
end;

//------------------------------------------------
function TVSL.vlm_RUNABOUT(AddOnId: word; V2LRec: pointer): TVAORC;
//------------------------------------------------
// proper responses: VAORC_SUCCESS | VAORC_GOTNOABOUT
begin
  result := VAORC_FAILURE;
  //---------------------------------------
  // Dispatch to appropriate AddOn
  //---------------------------------------
  try
    If CheckLimits(AddOnId,1,AddOnMax) and Assigned(FAddOns[AddOnId]) then
    Begin
      result := FAddOns[AddOnId].vlm_RUNABOUT(V2LRec);
    end else
    Begin
      MessageDlg('Attempt to RunAbout for AddOn ' + IntToStr(AddOnId), mtInformation, [mbOk], 0);
      result := VAORC_SUCCESS;
    end;
  except
  end;
end;

//------------------------------------------------
function TVSL.vlm_RUNHELP(AddOnId: word; V2LRec: pointer): TVAORC;
//------------------------------------------------
// proper response: VAORC_SUCCESS | VAORC_L2V_GOTNOHELP
begin
  result := VAORC_FAILURE;
  //---------------------------------------
  // Dispatch to appropriate AddOn
  //---------------------------------------
  try
    If CheckLimits(AddOnId,1,AddOnMax) and Assigned(FAddOns[AddOnId]) then
    Begin
      result := FAddOns[AddOnId].vlm_RUNHELP(V2LRec);
    end else
    Begin
      MessageDlg('Attempt to RunHelp for AddOn ' + IntToStr(AddOnId), mtInformation, [mbOk], 0);
      result := VAORC_SUCCESS;
    end;
  except
  end;
end;

//------------------------------------------------
function TVSL.vlm_UNLOAD(UndefinedW: word; UndefinedP: pointer): TVAORC;
//------------------------------------------------
// proper response: moot (but VAORC_SUCCESS is appreciated)
Var
  AddOnId: Word;
begin
  //---------------------------------------
  // Give Individual AddOns a chance
  //---------------------------------------
  try
    For AddOnId := 1 to AddOnMax do
    Begin
      If Assigned(FAddOns[AddOnId]) then
      Begin
        FAddOns[AddOnId].vlm_UNLOAD;
      end;
    end;
  except
  end;

  FvisApp := nil;
  result := VAORC_SUCCESS;
end;

//============================================================
//                        TAddOn
//============================================================

//------------------------------------------------
constructor TAddOn.Create(
      atts: TVAO_Atts;                   // Attributes of Add-on.
      enabMask: TVAO_EnabMask;           // Add-on's enabling policy.
      invokeOnMask: TVAO_InvokeMask;     // Invoke conditions
      notifyOnMask: TVAO_NotifyMask;     // notify conditions (none?)
      AName       : String;              // Add-on's name
      ADescr      : String               // Add-on's description
);
// If descendant overrides, call this first
//------------------------------------------------
begin
  RegRec.atts          := atts;
  RegRec.enabMask      := enabMask;
  RegRec.invokeOnMask  := invokeOnMask;     // Invoke conditions
  RegRec.notifyOnMask  := notifyOnMask;     // notify conditions (none?)
  FName                := AName;
  RegRec.lpName        := PChar(FName);
  Descr                := ADescr;
  Info       := TAddOnInfo.Create;
  RegRec.ord := VSL.EnlistAddOn(Self);
end;

//------------------------------------------------
function TAddOn.vlm_ENUMADDONS(V2LRec: PVAOV2LRecord): TVAORC;
//------------------------------------------------
// Generally don't need to override, this does the right thing
Begin
  result := VSL.RegisterAddon(PVAOV2LRecord(V2LRec)^.wSessID, @RegRec);
end;

//------------------------------------------------
function TAddOn.vlm_ISAOENABLED(V2LRec: PVAOV2LRecord): TVAORC;
//------------------------------------------------
// This is usually what you want
begin
  result := VAORC_L2V_ENABLED;
end;

//------------------------------------------------
function TAddOn.vlm_KILLSESSION(V2LRec: PVAOV2LRecord): TVAORC;
//------------------------------------------------
// Override this if AddOn needs to clean up on KillSession
begin

end;

//------------------------------------------------
function TAddOn.vlm_LOAD(InterfaceVersion: word; visApp: Visio_TLB.Application): TVAORC;
//------------------------------------------------
// Override this if AddOn needs to do something special on Load
begin

end;

//------------------------------------------------
function TAddOn.vlm_RUN(V2LRec: PVAOV2LRecord; Args: TStringList): TVAORC;
//------------------------------------------------
// Normally override this
begin
  Result := VAORC_SUCCESS;
end;

//------------------------------------------------
function TAddOn.vlm_RUNABOUT(V2LRec: PVAOV2LRecord): TVAORC;
//------------------------------------------------
// May override this
begin
  VSL.StandardAboutDialog;
  // Result := VAORC_L2V_GOTNOABOUT;
  Result := VAORC_SUCCESS;
end;

//------------------------------------------------
function TAddOn.vlm_RUNHELP(V2LRec: PVAOV2LRecord): TVAORC;
//------------------------------------------------
// May override this
begin
  Beep;
  Result := VAORC_L2V_GOTNOHELP;
end;

//------------------------------------------------
function TAddOn.vlm_UNLOAD: TVAORC;
//------------------------------------------------
// Override this if AddOn needs to do something special on UnLoad
begin
  Result := VAORC_SUCCESS;
end;

//------------------------------------------------------
procedure TAddOn.ErrorMessage(Msg: string);
//------------------------------------------------------
begin
  ShowMessage(Msg);
end;

//------------------------------------------------------
procedure TAddOn.InfoStr(var OutputStr: string; IndentSpaces: integer);
//------------------------------------------------------
begin
  OutputStr := OutputStr 
    + StringOfChar(' ', IndentSpaces)
    + '------------------------------------------------------' + CRLF

    + StringOfChar(' ', IndentSpaces  ) + 'Addon: ' + Name                 + CRLF
    + StringOfChar(' ', IndentSpaces+2) + Descr                            + CRLF

    + StringOfChar(' ', IndentSpaces)
    + '------------------------------------------------------' + CRLF;

  Info.InfoStr(OutputStr, IndentSpaces + 2);
end;

initialization
  VSL := TVSL.Create;
end.

