Unit DAVSL_Util;
{---------------------------------------------------------------
Part of Diagramantics VSL Framework (DAVSL)
www.diagramantics.com

TAddOnInfo, TArgInfo and TValInfo allow setting Value and Argument info so
that:
-- Argument string values can be decoded to integers
-- The About box can provide meaningful info on how to use AddOn.

See Examples for how to set up these objects (in YourAddon.Create), and use
them (in RUN functions).

TAppVerInfo object adapted from:
Author: Riccardo "Rico" Pareschi
E-Mail: rico@mbox.4net.it

Version History
----------------
2000-09-17 GW: Original
---------------------------------------------------------------}

//-----------------------------------------------------------
                        interface
//-----------------------------------------------------------
uses Windows;
Type
  //---------------------------
  TValInfo = class
  //---------------------------
    Name        : string;
    Id          : integer;
    Descr       : string;
    constructor Create(AName: string; AId: integer; ADescr: string);
    procedure InfoStr(var OutputStr: string; IndentSpaces: integer);
  end;

  //---------------------------
  TArgInfo = class
  //---------------------------
    Name        : string;
    Id          : integer;
    Descr       : string;
    ValCaseSens : boolean;
    DefaultVal  : integer;

    ValCount    : integer;
    Vals        : array of TValInfo;

    constructor Create(AName: string; AId: integer; ADescr: string;
                       AValCaseSens: boolean; ADefaultVal: integer);
    destructor Destroy;
    function AddValInfo(AName: string; AId: integer; ADescr: string): TValInfo;
    function GetValInfo(AValName: string): TValInfo;
    function LookupVal(AValName: string): integer;
    procedure InfoStr(var OutputStr: string; IndentSpaces: integer);
  end;

  //---------------------------
  TAddOnInfo = class
  //---------------------------
    // Name        : string;
    // Id          : integer;
    Descr       : string;

    ArgCount    : integer;
    Args        : array of TArgInfo;

    constructor Create;
    destructor Destroy;
    function AddArgInfo(AName: string; AId: integer; ADescr: string;
                        AValCaseSens: boolean; ADefaultVal: integer):TArgInfo;
    function GetArgInfo(AArgName: string): TArgInfo;
    function LookupArg(AArgName: string): integer;
    procedure InfoStr(var OutputStr: string; IndentSpaces: integer);
  end;

  //---------------------------
  TAppVerInfo = class
  //---------------------------
  private
    { Private declarations }
    FFilePath: string;
    FVSLName: string;

    Size  : DWord;
    VSize : DWord;
    VData : Pointer;
    VVers : Pointer;
    Len   : DWord;
    FCompanyName: string;
    FFileDescription: string;
    FFileVersion: string;
    FLegalCopyright: string;
    FOriginalFilename: string;
    FComments: string;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create;
    function    GetVerValue(Value : string): string;
    procedure   ExtractVerInfo;

    property FilePath           : string read FFilePath;
    property VSLName            : string read FVSLName;

    property CompanyName        : string read FCompanyName;
    property FileDescription    : string read FFileDescription;
    property FileVersion        : string read FFileVersion;
    property LegalCopyright     : string read FLegalCopyright;
    property OriginalFilename   : string read FOriginalFilename;
    property Comments           : string read FComments;
  end;
Const
  CRLF = #13#10;

Var
  AppVerInfo: TAppVerInfo;

//-----------------------------------------------------------
                        implementation
//-----------------------------------------------------------
uses SysUtils, Forms;

{========================== TValInfo ===================================}

//---------------------------------------------------------------------------
constructor TValInfo.Create(AName: string; AId: integer;  ADescr: string);
//---------------------------------------------------------------------------
begin
  Name  := AName ;
  Id    := AId;
  Descr := ADescr;
end;

//---------------------------------------------------------------------------
procedure TValInfo.InfoStr(var OutputStr: string; IndentSpaces: integer);
//---------------------------------------------------------------------------
begin
  OutputStr := OutputStr + StringOfChar(' ', IndentSpaces)
               + Name + ' = '+IntToStr(Id)+' "'+Descr + '"' + CRLF;
end;

{============================= TArgInfo ==============================}

//---------------------------------------------------------------------------
constructor TArgInfo.Create(AName: string; AId: integer; ADescr: string;
                            AValCaseSens: boolean; ADefaultVal: integer);
//---------------------------------------------------------------------------
begin
  ValCount    := 0;
  Name        := AName     ;
  Id           := AId    ;
  Descr       := ADescr    ;
  ValCaseSens    := AValCaseSens ;
  DefaultVal  := ADefaultVal;
end;

//---------------------------------------------------------------------------
destructor TArgInfo.Destroy;
//---------------------------------------------------------------------------
Var
  Ix: integer;
begin
  For Ix := 0 to ValCount - 1 do Vals[Ix].Free;
end;

//---------------------------------------------------------------------------
function TArgInfo.AddValInfo(AName: string; AId: integer; ADescr: string): TValInfo;
//---------------------------------------------------------------------------
begin
  If Length(Vals) <= ValCount+1 then SetLength(Vals, ValCount + 5);
  Vals[ValCount] := TValInfo.Create(AName, AId, ADescr);
  Result := Vals[ValCount];
  Inc(ValCount);
end;

//---------------------------------------------------------------------------
procedure TArgInfo.InfoStr(var OutputStr: string; IndentSpaces: integer);
//---------------------------------------------------------------------------
Var
  Ix: integer;
  S: string;
begin
  S := 'CI';
  If ValCaseSens then S := 'CS';

  OutputStr := OutputStr
    + StringOfChar(' ', IndentSpaces) + Name + ' = '+IntToStr(Id)
      + ' "' + Descr + '" ' + CRLF
    + StringOfChar(' ', IndentSpaces+2) + 'Values (' + S + ')' + CRLF;

  For Ix := 0 to ValCount - 1 do Vals[Ix].InfoStr(OutputStr, IndentSpaces + 2);
end;

//---------------------------------------------------------------------------
function TArgInfo.GetValInfo(AValName: string): TValInfo;
//---------------------------------------------------------------------------
Var
  Ix: integer;
  VI: TValInfo;
Label done;
begin
  Result := nil;
  For Ix := 0 to ValCount - 1 do
  Begin
    VI := Vals[Ix];
    If ((not ValCaseSens) and (CompareText(AValName, VI.Name)=0)) or
       (     ValCaseSens  and (AValName = VI.Name)) then
    Begin
      Result := VI;
      goto done;
    end;
  end;
done:
end;

//---------------------------------------------------------------------------
function TArgInfo.LookupVal(AValName: string): integer;
//---------------------------------------------------------------------------
Var
  VI: TValInfo;
begin
  Result := DefaultVal;
  VI := GetValInfo(AValName);
  If Assigned(VI) then Result := VI.Id;
end;


{=========================== TAddOnInfo ===============================}

//---------------------------------------------------------------------------
constructor TAddOnInfo.Create;
//---------------------------------------------------------------------------
begin
  // Id          := 0;
  // Name        := '';
  Descr       := '';
  ArgCount    := 0;
end;

//---------------------------------------------------------------------------
destructor TAddOnInfo.Destroy;
//---------------------------------------------------------------------------
Var
  Ix: integer;
begin
  For Ix := 0 to ArgCount - 1 do Args[Ix].Free;
end;

//---------------------------------------------------------------------------
function TAddOnInfo.AddArgInfo(AName: string; AId: integer; ADescr: string;
                               AValCaseSens: boolean; ADefaultVal: integer): TArgInfo;
//---------------------------------------------------------------------------
Begin
  If Length(Args) <= ArgCount+1 then
    SetLength(Args, ArgCount + 5);
  Args[ArgCount] := TArgInfo.Create(AName, AId, ADescr, AValCaseSens, ADefaultVal);
  Result := Args[ArgCount];
  Inc(ArgCount);
end;

//---------------------------------------------------------------------------
procedure TAddOnInfo.InfoStr(var OutputStr: string; IndentSpaces: integer);
//---------------------------------------------------------------------------
Var
  Ix: integer;
begin
  If ArgCount > 0 then
  Begin
    OutputStr := OutputStr
      + StringOfChar(' ', IndentSpaces) + 'Arguments ' + CRLF ;
    For Ix := 0 to ArgCount - 1 do Args[Ix].InfoStr(OutputStr, IndentSpaces + 2);
  end;
end;

//---------------------------------------------------------------------------
function TAddOnInfo.GetArgInfo(AArgName: string): TArgInfo;
//---------------------------------------------------------------------------
Var
  Ix: integer;
  AI: TArgInfo;
Label done;
begin
  Result := nil;
  For Ix := 0 to ArgCount - 1 do
  Begin
    AI := Args[Ix];
    If (CompareText(AArgName, AI.Name)=0) then
    Begin
      Result := AI;
      goto done;
    end;
  end;
done:
end;

//---------------------------------------------------------------------------
function TAddOnInfo.LookupArg(AArgName: string): integer;
//---------------------------------------------------------------------------
Var
  AI: TArgInfo;
begin
  Result := -1;
  AI := GetArgInfo(AArgName);
  If Assigned(AI) then Result := AI.Id;
end;

//==========================================================
// TAppVerInfo
//==========================================================

//-----------------------
constructor TAppVerInfo.Create;
//-----------------------
  procedure PathToVSLName;
  Var
    S: string;
    P: integer;
  Begin
    S := ExtractFileName(FFilePath);
    P := Pos('.', S);
    If P <> 0 then S := Copy(S, 1, P-1);
    FVSLName := S;
  end;
Var
  S : array[0..1023] of char;
begin
  inherited;
  (* Commented out: the Application version
  FFilePath := Application.ExeName;
  FVSLName := GetVSLName;
  *)
  GetModuleFileName(hInstance, S, 1023);
  FFilePath := StrPas(S);
  PathToVSLName;
  ExtractVerInfo;
end; {- Create }

//--------------------------------------------------------
function TAppVerInfo.GetVerValue(Value : string): string;
//--------------------------------------------------------
var LangID: string;
begin
  LangID := Format('%.4x',[GetSystemDefaultLangID])+'04E4';
  if VerQueryValue(VData,
                   pChar(Format('\StringFileInfo\%s\%s', [LangID, Value])),
                   VVers, Len) then
     if Len > 0 then begin
        Result := StrPAs(VVers);
     end;
end; {- GetVerValue }


{-------------------------------------------------------------------------------
  This is all the job!
  There are a lot of informations to read from the file version. But I think
  these are the most useful for the about box.
}
procedure TAppVerInfo.ExtractVerInfo;
begin
  VSize := GetFileVersionInfoSize(Pchar(FFilePath), Size);
  if VSize <> 0 then begin
     GetMem(VData, VSize);
     try
       if GetFileVersionInfo(Pchar(FFilePath), 0, VSize, VData) then
          begin
            FCompanyName       := GetVerValue('CompanyName');
            FFileDescription   := GetVerValue('FileDescription');
            FFileVersion       := GetVerValue('FileVersion');
            FLegalCopyright    := GetVerValue('LegalCopyright');
            FOriginalFilename  := GetVerValue('OriginalFilename');
            FComments          := GetVerValue('Comments');
          end;
     finally
       FreeMem(VData, VSize);
     end; // try...Finally
  end; // if Vsize
end; {- ExtractVerInfo }

initialization
  AppVerInfo := TAppVerInfo.Create;
end.

